@@ -114,8 +114,8 @@ static void tcc_add_systemdir(TCCState *s)
 }
 #endif
 
-#ifndef CONFIG_TCC_STATIC
-void dlclose(void *p)
+#if !defined(CONFIG_TCC_STATIC) || (defined(CONFIG_TCC_STATIC) && defined(CONFIG_TCC_STATIC_NODLOPEN))
+static void dlclose(void *p)
 {
     FreeLibrary((HMODULE)p);
 }
