@@ -57,10 +57,14 @@ namespace eval tcc4tcl {
 		return $handle
 	}
 
-	proc _linktclcommand {handle cSymbol tclCommand} {
+	proc _linktclcommand {handle cSymbol args} {
 		upvar #0 $handle state
+		set argc [llength $args]
+		if {$argc != 1 && $argc != 2} {
+			return -code error "_linktclcommand handle cSymbol tclCommand ?clientData?"
+		}
 
-		lappend state(procs) $cSymbol $tclCommand
+		lappend state(procs) $cSymbol $args
 	}
 
 	proc _add_include_path {handle args} {
@@ -93,7 +97,7 @@ namespace eval tcc4tcl {
 		append state(code) $wrapped "\n"
 		append state(code) $wrapper "\n"
 
-		lappend state(procs) $name $tclname
+		lappend state(procs) $name [list $tclname]
 	}
 
 	proc _cproc {handle name adefs rtype {body "#"}} {
@@ -108,7 +112,7 @@ namespace eval tcc4tcl {
 		append state(code) $wrapped "\n"
 		append state(code) $wrapper "\n"
 
-		lappend state(procs) $name $tclname
+		lappend state(procs) $name [list $tclname]
 	}
 
 	proc _ccode {handle code} {
@@ -406,8 +410,16 @@ namespace eval tcc4tcl {
 				# No additional code needed
 				if {$outputOnly} {
 					if {[info exists state(procs)] && [llength $state(procs)] > 0} {
-						foreach {procname cname} $state(procs) {
-							append code "/* Immediate: Tcl_CreateObjCommand(interp, \"$procname\", $cname, NULL, NULL); */\n"
+						foreach {procname cname_obj} $state(procs) {
+							set cname [lindex $cname_obj 0]
+
+							if {[llength $cname_obj] > 1} {
+								set obj [lindex $cname_obj 1]
+							} else {
+								set obj "NULL"
+							}
+
+							append code "/* Immediate: Tcl_CreateObjCommand(interp, \"$procname\", $cname, $obj, Tcc4tclDeleteClientData); */\n"
 						}
 					}
 				}
@@ -416,7 +428,13 @@ namespace eval tcc4tcl {
 				if {[info exists state(procs)] && [llength $state(procs)] > 0} {
 					append code "int _initProcs(Tcl_Interp *interp) \{\n"
 					
-					foreach {procname cname} $state(procs) {
+					foreach {procname cname_obj} $state(procs) {
+						set cname [lindex $cname_obj 0]
+
+						if {[llength $cname_obj] != 1} {
+							error "ClientData not supported in exe / dll mode"
+						}
+
 						append code "  Tcl_CreateObjCommand(interp, \"$procname\", $cname, NULL, NULL);\n"
 					}
 
@@ -438,7 +456,13 @@ namespace eval tcc4tcl {
 				append code "#endif\n"
 
 				if {[info exists state(procs)] && [llength $state(procs)] > 0} {
-					foreach {procname cname} $state(procs) {
+					foreach {procname cname_obj} $state(procs) {
+						set cname [lindex $cname_obj 0]
+
+						if {[llength $cname_obj] != 1} {
+							error "ClientData not supported in exe / dll mode"
+						}
+
 						append code "  Tcl_CreateObjCommand(interp, \"$procname\", $cname, NULL, NULL);\n"
 					}
 				}
@@ -486,8 +510,8 @@ namespace eval tcc4tcl {
 				tcc compile $code
 
 				if {[info exists state(procs)] && [llength $state(procs)] > 0} {
-					foreach {procname cname} $state(procs) {
-						tcc command $procname $cname
+					foreach {procname cname_obj} $state(procs) {
+						tcc command $procname {*}$cname_obj
 					}
 				}
 			}
@@ -566,13 +590,27 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""} {includePrototype 0
 	set code {}
 
 	# Write wrapper
-	append cbody "int $wname\(ClientData dummy, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv\[\]) {" "\n"
+	append cbody "int $wname\(ClientData clientdata, Tcl_Interp *ip, int objc, Tcl_Obj *CONST objv\[\]) {" "\n"
 
 	# if first arg is "Tcl_Interp*", pass it without counting it as a cmd arg
-	if {[lindex $adefs 0] eq "Tcl_Interp*"} {
-		lappend cnames ip
-		lappend cargs [lrange $adefs 0 1]
-		set adefs [lrange $adefs 2 end]
+	while {1} {
+		if {[lindex $adefs 0] eq "Tcl_Interp*"} {
+			lappend cnames ip
+			lappend cargs [lrange $adefs 0 1]
+			set adefs [lrange $adefs 2 end]
+
+			continue
+		}
+
+		if {[lindex $adefs 0] eq "ClientData"} {
+			lappend cnames clientdata
+			lappend cargs [lrange $adefs 0 1]
+			set adefs [lrange $adefs 2 end]
+
+			continue
+		}
+
+		break
 	}
 
 	foreach {t n} $adefs {
@@ -617,6 +655,7 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""} {includePrototype 0
 	# Create wrapper function
 	## Supported input types
 	##   Tcl_Interp*
+	##   ClientData
 	##   int
 	##   long
 	##   float
@@ -739,4 +778,4 @@ proc ::tcc4tcl::wrap {name adefs rtype {body "#"} {cname ""} {includePrototype 0
 
 namespace eval tcc4tcl {namespace export cproc new}
 
-package provide tcc4tcl "0.23"
+package provide tcc4tcl "0.24"
